@@ -3,6 +3,8 @@ from urllib import unquote
 from urlparse import urljoin
 from datetime import datetime
 from sha import sha
+from collections import defaultdict
+from threading import Lock
 
 from webob import Request, Response
 from simplejson import loads, dumps, JSONEncoder
@@ -25,6 +27,7 @@ class DatetimeEncoder(JSONEncoder):
 class JSONStore(object):
     def __init__(self, **kwargs):
         self.em = EntryManager(**kwargs)
+        self.lock = defaultdict(Lock)
 
     def __call__(self, environ, start_response):
         req = Request(environ)
@@ -99,16 +102,17 @@ class JSONStore(object):
         elif url_id != entry['__id__']:
             return Response(status='409 Conflict')
 
-        # Conditional PUT.
-        old = self.em.search(__id__=url_id)
-        if old:
-            etag = '"%s"' % sha(dumps(old[0], cls=DatetimeEncoder)).hexdigest()
-            if etag not in req.if_match or (
-                    req.if_unmodified_since and 
-                    req.if_unmodified_since < old[0]['updated']):
-                return Response(status='412 Precondition Failed')
+        # Conditional PUT. This is useful for implementing counters, eg.
+        with self.lock[url_id]:
+            old = self.em.search(__id__=url_id)
+            if old:
+                etag = '"%s"' % sha(dumps(old[0], cls=DatetimeEncoder)).hexdigest()
+                if etag not in req.if_match or (
+                        req.if_unmodified_since and 
+                        req.if_unmodified_since < old[0]['updated']):
+                    return Response(status='412 Precondition Failed')
+            result = self.em.update(entry)
 
-        result = self.em.update(entry)
         body = dumps(result, cls=DatetimeEncoder)
         etag = '"%s"' % sha(body).hexdigest()
 
