@@ -4,15 +4,10 @@ import operator
 from datetime import datetime
 import time
 import re
+import sqlite3
 import threading
 LOCAL = threading.local()
 
-try:
-    import sqlite3 as sqlite
-except ImportError:
-    from pysqlite2 import dbapi2 as sqlite
-
-from shove import Shove
 from uuid import uuid4
 
 from jsonstore.operators import Operator, Equal
@@ -29,13 +24,10 @@ def regexp(expr, item):
 
 
 class EntryManager(object):
-    def __init__(self, store='simple://', cache='simple://', index='index.db', **kwargs):
-        self.store = Shove(store, cache, **kwargs)
-
+    def __init__(self, index='index.db', **kwargs):
         self.index = index
         if not os.path.exists(self.index):
             self._create_table()
-            self.reindex()
 
     # Thread-safe connection manager. Conections are stored in the 
     # ``threading.local`` object, so they can be safely reused in the
@@ -46,16 +38,15 @@ class EntryManager(object):
             LOCAL.conns = {}
 
         if self.index not in LOCAL.conns:
-            LOCAL.conns[self.index] = sqlite.connect(self.index, 
-                    detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
-        LOCAL.conns[self.index].create_function("regexp", 2, regexp)
+            LOCAL.conns[self.index] = sqlite3.connect(self.index, 
+                    detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)
+        LOCAL.conns[self.index].create_function("REGEXP", 2, regexp)
         return LOCAL.conns[self.index]
 
     def _create_table(self):
         self.conn.executescript("""
             CREATE TABLE flat (
                 id VARCHAR(255),
-                updated timestamp,
                 position CHAR(255),
                 leaf NUMERIC);
             CREATE INDEX position ON flat (position);
@@ -71,21 +62,19 @@ class EntryManager(object):
 
         # __id__ and __updated__ can be overriden.
         id_ = entry.setdefault('__id__', str(uuid4()))
-        if id_ in self.store:
-            raise ConflictError('Conflict!')
+        curs = self.conn.execute("SELECT id FROM flat WHERE id=?", (id_,))
+        if curs.fetchall():
+            raise ConflictError('Conflict, the id "%s" already exists!' % id_)
         updated = entry.setdefault('__updated__', datetime.utcnow())
         if not isinstance(updated, datetime):
             updated = datetime(
                 *(time.strptime(updated, '%Y-%m-%dT%H:%M:%SZ')[0:6]))
 
         # Store entry.
-        self.store[id_] = entry
         self.index_entry(entry)
         return entry
 
     def delete(self, key):
-        del self.store[key]
-
         self.conn.execute("""
             DELETE FROM flat
             WHERE id=?;
@@ -161,7 +150,7 @@ class EntryManager(object):
             query.append("(%s)" % " OR ".join(condition))
         if leaves:
             query.append("GROUP BY id HAVING COUNT(*)=%d" % leaves)
-        query.append("ORDER BY updated DESC")
+        #query.append("ORDER BY updated DESC")
         if size is not None:
             query.append("LIMIT %s" % size)
         if offset:
@@ -174,24 +163,37 @@ class EntryManager(object):
             return curs.fetchone()[0]
         else:
             curs = self.conn.execute(query, tuple(params))
-            return [self.store[row[0]] for row in curs]
+            return [ self.load_entry(row[0]) for row in curs ]
+
+    def load_entry(self, id_):
+        entry = { '__id__': id_ }
+        curs = self.conn.execute("""
+            SELECT position, leaf FROM flat WHERE id=?
+        """, (id_,))
+        for position, leaf in curs.fetchall():
+            target = entry
+            tokens = position.split('.')
+            for token in tokens[:-1]:
+                if token not in target:
+                    target[token] = {}
+                target = target[token]
+            if tokens[-1] in target:
+                target[ tokens[-1] ] = [ target[ tokens[-1] ], leaf ]
+            else:
+                target[ tokens[-1] ] = leaf
+
+        return entry
 
     def index_entry(self, entry):
         # Index entry.
-        indexes = [(entry['__id__'], entry['__updated__'], k, v)
+        indexes = [(entry['__id__'], k, v)
                 for (k, v) in flatten(entry) if k != '__id__']
         self.conn.executemany("""
-            INSERT INTO flat (id, updated, position, leaf)
-            VALUES (?, ?, ?, ?);
+            INSERT INTO flat (id, position, leaf)
+            VALUES (?, ?, ?);
         """, indexes)
         self.conn.commit()
 
-    def reindex(self):
-        self.conn.execute("DELETE FROM flat;")
-        self.conn.commit()
-        for entry in self.store:
-            self.index_entry(self.store[entry])
-        
     def close(self):
         self.conn.close()
         del LOCAL.conns[self.index]
